/*
 * dvbstub1.h
 *
 * Copyright (C) 2009 Sergey Mironov <ierton@gmail.com>
 *
 * based on pluto2.c 1.10 - http://instinct-wp8.no-ip.org/pluto/
 * 	by Dany Salman <salmandany@yahoo.fr>
 *	Copyright (c) 2004 TDF
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 

#ifndef DVBSTUB1_H_INCLUDED
#define DVBSTUB1_H_INCLUDED

//#include "dvb_frontend.h"


#define DRIVER_NAME	"dvbstub1"

#define DVBSTUB1_DEBUG 
//#define DVBSTUB_OLD_KERNEL

#define ERR(args...) \
	do{ printk(KERN_WARNING DRIVER_NAME ":%s(): ", __func__);  printk(args); printk("\n"); } while(0);

#define INFO(args...) \
	do{ printk(KERN_INFO DRIVER_NAME ":%s(): ", __func__);  printk(args); printk("\n"); } while(0);


#ifdef DVBSTUB1_DEBUG

#define DBG(args...) \
	do{ printk(KERN_INFO DRIVER_NAME ":%s(): ", __func__);  printk(args); printk("\n"); } while(0);
//    do{ printk(KERN_INFO DRIVER_NAME ":" THIS_FUNC  "(): " args); printk("\n"); } while(0);

#else

#define DBG(args...)

#endif


/********************************************************************
 * INPUT 
 ********************************************************************/

struct dvbstub1_input_ops
{
	/** 
	 * @brief Name of device
	 */
	const char* name;

	unsigned long baud_rate;

	/** 
	 * @brief Callback to handle write request for device
	 * 
	 * @param devid Minor number of device
	 * @param buf Pointer to user data
	 * @param sz Size of buffer
	 * @returns Error code or zero
	 */
	int (*write) (int devid, const char *buf, size_t sz);

	/** 
	 * @brief Callback to handle device read requests
	 * 
	 * @param devid Minor number of device
	 * @param buf 
	 * @param sz Max size of buffer
	 * @retval 0 - EOF
	 * @returns Number of bytes written (if >0) or error code (if <0)
	 */
	int (*read) (int devid, char *buf, size_t sz);
};

/** 
 * @brief Creates dcount of char devices for test dvb input
 * 
 * @param dcount Number of input devices to initt
 * 
 * @retval 0 Success
 * @retval Non_null_value Error code
 */
int __init dvbstub1_input_init(int dcount, struct dvbstub1_input_ops *pops);

/** 
 * @brief Cleanups devices
 */
void __exit dvbstub1_input_uninit(void);

/********************************************************************
 * FRONTEND 
 ********************************************************************/
struct dvb_frontend;

int dvbstub1_fe_init(struct dvb_frontend *fe);

void dvbstub1_fe_uninit(struct dvb_frontend *fe);

/********************************************************************
 * BAUDCALC 
 ********************************************************************/

struct baudcalc {

	unsigned long tstart_jiffie;
	unsigned long speed_bytes_per_jiffie;
	unsigned long data_size_bytes;
	unsigned long capacity;

	/*TODO: one must add spinlock here*/
};

void baudcalc_init(struct baudcalc* bc, unsigned long speed_bytes_per_jf);

void baudcalc_reset(struct baudcalc* bc);

int baudcalc_test(struct baudcalc* bc, unsigned long now, unsigned long data_size, unsigned long* delay);

void baudcalc_add(struct baudcalc* bc, unsigned long data_size);


#ifdef BAUDCALC_TEST
void baudcalc_set_jiffie(unsigned long new_jiffie);
#endif

#endif

