/*
 * dvbstub1_baudc.c
 *
 * Copyright (C) 2009 Sergey Mironov <ierton@gmail.com>
 *
 * based on pluto2.c 1.10 - http://instinct-wp8.no-ip.org/pluto/
 * 	by Dany Salman <salmandany@yahoo.fr>
 *	Copyright (c) 2004 TDF
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/time.h>
#include <linux/timer.h>
#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <linux/types.h>
#include <linux/spinlock.h>
#include <linux/interrupt.h>

#include "dvbstub1.h"

#ifdef BAUDCALC_TEST
#define BAUDCALC_JIFFIES jfake;
static unsigned long jfake;
void baudcalc_set_jiffie(unsigned long new_jiffie)
{
	jfake = new_jiffie;
}
#else
#define BAUDCALC_JIFFIES jiffies;
#endif

#if 0
void baudcalc_init(struct baudcalc* bc, unsigned long speed_bytes_per_msec)
{
	bc->data_size_bytes = 0;
	bc->tstart_jiffie = BAUDCALC_JIFFIES;
	bc->speed_bytes_per_jiffie = (speed_bytes_per_msec * 1000) / HZ;
}
#endif

void baudcalc_init(struct baudcalc* bc, unsigned long speed_bytes_per_jf)
{
	bc->speed_bytes_per_jiffie = speed_bytes_per_jf;
	baudcalc_reset(bc);
}

void baudcalc_reset(struct baudcalc* bc)
{
	bc->data_size_bytes = 0;
	bc->tstart_jiffie = 0;
}

unsigned long cont_block = 0;
int baudcalc_test(struct baudcalc* bc, unsigned long now, unsigned long data_size, unsigned long* delay)
{
	register unsigned long t;

	if(0 == bc->tstart_jiffie) {
		DBG("Initializing baudcalc");
		bc->tstart_jiffie = now-1;
		bc->capacity = 0;
	}

	bc->capacity = (now - bc->tstart_jiffie) * bc->speed_bytes_per_jiffie;

	if(bc->capacity > bc->data_size_bytes) {
		return min(data_size, bc->capacity - bc->data_size_bytes);
	}

/*    DBG("Capacity:%lu, data_size:%lu, new_data:%lu, Elapsed time:%lu, last_continuous blocl:%lu", */
/*        bc->capacity, bc->data_size_bytes, data_size, now-bc->tstart_jiffie, cont_block);*/
	cont_block = 0;

	t = ((bc->data_size_bytes + data_size) / bc->speed_bytes_per_jiffie) + 1;
	*delay = bc->tstart_jiffie + t - now;
	return 0;
}

void baudcalc_add(struct baudcalc* bc, unsigned long data_size)
{
	cont_block+=data_size;
	bc->data_size_bytes += data_size;
/*    DBG("Adding %lu bytes to baudcalc", data_size);*/
}




