/*
 * dvbstub1.c
 *
 * Copyright (C) 2009 Sergey Mironov <ierton@gmail.com>
 *
 * based on pluto2.c 1.10 - http://instinct-wp8.no-ip.org/pluto/
 * 	by Dany Salman <salmandany@yahoo.fr>
 *	Copyright (c) 2004 TDF
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/vmalloc.h>
#include <linux/cdev.h>

#include "demux.h"
#include "dmxdev.h"
#include "dvb_demux.h"
#include "dvb_frontend.h"
#include "dvbdev.h"
#include "dmxdev.h"
	
#include "dvbstub1.h"

#include "dvbstub1_baudc.c"

#define REQUIRE(x, label)  						\
	do{ if(!(x)) { 								\
		ERR("ERROR: Condition '%s' on %s:%d failed", #x, __FILE__, __LINE__); \
		goto label; 							\
	} } while(0)

#define REQUIRE_MESSAGE(x, label, arg...)  		\
	do{ if(!(x)) { 								\
		ERR("ERROR: Condition '%s' failed with message:", #x);\
		ERR(arg);								\
		goto label; 							\
	} } while(0)


struct baudcalc  g_baudc;

void baudcalc_print(struct baudcalc* bc)
{
	INFO("bc->tstart_jiffie = %lu", bc->tstart_jiffie);
	INFO("bc->data_size_bytes = %lu", bc->data_size_bytes );
	INFO("bc->speed_bytes_per_jiffie = %lu", bc->speed_bytes_per_jiffie);
}


int baudc_test1(unsigned long speed, unsigned long data)
{
	struct baudcalc* bc = &g_baudc;
	unsigned long test_speed = speed;
	unsigned long test_data;
	unsigned long test_tstart = 100;
	unsigned long res, wait_until = 0;
	INFO("Executing with speed=%lu, data_size=%lu", speed, data);

	baudcalc_init(bc, test_speed);

	REQUIRE_MESSAGE(bc->tstart_jiffie == 0, err, 
		"Bad start time : %lu, should be 0", bc->tstart_jiffie);

	REQUIRE_MESSAGE(bc->data_size_bytes == 0, err, 
		"Bad data size: %lu", bc->data_size_bytes );

	REQUIRE_MESSAGE(bc->speed_bytes_per_jiffie == test_speed, err, 
		"Bad rate size: %lu", bc->speed_bytes_per_jiffie);

	test_data = data;
	res = baudcalc_test(bc, test_tstart, test_data, &wait_until);
	INFO("Calulatyed capacity: %lu", bc->capacity);

	REQUIRE(wait_until == 0, err);

	if(speed > data)
		REQUIRE(res == data, err);
	else
		REQUIRE(res == speed, err);

	baudcalc_add(bc, speed);
	res = baudcalc_test(bc, test_tstart, test_data, &wait_until);
	REQUIRE(res == 0, err);
	REQUIRE(wait_until > 0, err);

	INFO("Calculated wakeup delay: %lu", wait_until);

	res = baudcalc_test(bc, test_tstart + wait_until, test_data, &wait_until);

	REQUIRE_MESSAGE(res == test_data, err, 
		"Bad result. Should be %lu, but actually: %lu", test_data, res);

	return 0;

err:
	return -1;
};


#if 0
int baudc_test2(void)
{
	struct baudcalc* bc = &g_baudc;
	unsigned long test_speed = 80;
	unsigned long test_data;
	unsigned long test_tstart = 100;
	unsigned long res, wait_until = 0;

	baudcalc_set_jiffie(test_tstart);

	baudcalc_init(bc, test_speed);

	baudcalc_print(bc);


	test_data = 20;
	res = baudcalc_test(bc, test_data, &wait_until);
	REQUIRE(res == 0, err);
	REQUIRE_MESSAGE(wait_until == test_data / test_speed  + test_tstart, err,
		"Bad wait_until: %lu. Should be: %lu", wait_until, test_speed / test_speed + test_tstart);

	INFO("wait_until == %lu", wait_until);
	baudcalc_set_jiffie(wait_until);
	test_data = 40;
	res = baudcalc_test(bc, test_data, &wait_until);
/*    REQUIRE_MESSAGE(res == 20, err, */
/*        "Bad result. Should be 20, but actually: %lu", res);*/
	INFO("Wait result == %lu", res);

	return 0;

err:
	return -1;
};
#endif

static int __init dvbstub1_baudc_test_init(void)
{
	INFO("HZ=%d", HZ);
	REQUIRE( 0 == baudc_test1(10000,100), err);
	REQUIRE( 0 == baudc_test1(1000,1000), err);
	REQUIRE( 0 == baudc_test1(100,10000), err);
	REQUIRE( 0 == baudc_test1(10,100000), err);

/*    REQUIRE( 0 == baudc_test2(), err);*/

	INFO("completed");
	return -EAGAIN; /* means success */

err:
	return -1;
}

static void __exit dvbstub1_baudc_test_exit(void)
{
}

module_init(dvbstub1_baudc_test_init);
module_exit(dvbstub1_baudc_test_exit);


