/*
 * dvbstub1.c
 *
 * Copyright (C) 2009 Sergey Mironov <ierton@gmail.com>
 *
 * based on pluto2.c 1.10 - http://instinct-wp8.no-ip.org/pluto/
 * 	by Dany Salman <salmandany@yahoo.fr>
 *	Copyright (c) 2004 TDF
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/vmalloc.h>
#include <linux/cdev.h>

#include "demux.h"
#include "dmxdev.h"
#include "dvb_demux.h"
#include "dvb_frontend.h"
#include "dvbdev.h"
#include "dmxdev.h"
	
#include "dvbstub1.h"



#ifndef DVBSTUB_OLD_KERNEL
DVB_DEFINE_MOD_OPT_ADAPTER_NR(g_adapter_nums);
#endif

struct dvbstub1 {
	/* dvb */
	struct dvb_adapter adapter;
	struct dvb_demux demux;
	struct dmxdev dmxdev;

	struct dvb_frontend frontend;
	struct dmx_frontend frontend_mem;
	struct dmx_frontend frontend_hw;

	unsigned int users;
};

struct dvbstub1* g_dvbstub1 = NULL;

int g_baud_rate = 2000;

int dvbstub1_input_write_cb(int devid, const char *buf, size_t sz)
{
/*    static size_t total = 0;*/
/*    unsigned long j1 = jiffies+g_delay;*/
/*    while (time_before(jiffies, j1))*/
/*        cpu_relax();*/
/*        schedule();*/

	// DBG("entering. devid=%d, size=%d, totalsize=%d Kb", devid, sz, (total+=sz)/1024 );
	dvb_dmx_swfilter(&g_dvbstub1->demux, buf, sz);
	// DBG("leaving");
	return sz;
}


int dvbstub1_input_read_cb(int devid, char *buf, size_t sz)
{
	ERR("Not implemented");
	return 0;
}

struct dvbstub1_input_ops g_dvbstub_input_ops = {
	.name = "dvbstubi",
	.baud_rate = 2000,
	.read = dvbstub1_input_read_cb,
	.write = dvbstub1_input_write_cb
};

static inline struct dvbstub1 *feed_to_dvbstub1(struct dvb_demux_feed *feed)
{
	return container_of(feed->demux, struct dvbstub1, demux);
}

static inline struct dvbstub1 *frontend_to_dvbstub1(struct dvb_frontend *fe)
{
	return container_of(fe->dvb, struct dvbstub1, adapter);
}

static int dvbstub1_start_feed(struct dvb_demux_feed *f)
{
	INFO("pid=%d", f->pid);

	return 0;
}

static int dvbstub1_stop_feed(struct dvb_demux_feed *f)
{
	INFO("pid=%d", f->pid);

	return 0;
}

static int __init dvbstub1_init(void)
{
	struct dvbstub1 *device;
	int ret = 0;

	DBG("entering");

	g_dvbstub_input_ops.baud_rate = (unsigned long)(g_baud_rate*1000)/HZ;
	INFO("Configured with baud_rate=%d bytes/msec (%lu jiffies/msec) HZ=%d", 
		g_baud_rate, g_dvbstub_input_ops.baud_rate, HZ);

	device = kzalloc( sizeof(struct dvbstub1), GFP_KERNEL);
	if (!device) {
		ret = -ENOMEM;
		ERR("kzalloc() failed. ret = %d", ret);
		goto out;
	}

	/* dvb - adapter */
#ifndef DVBSTUB_OLD_KERNEL
	ret = dvb_register_adapter(&device->adapter, DRIVER_NAME, THIS_MODULE, NULL, g_adapter_nums);
#else
	ret = dvb_register_adapter(&device->adapter, DRIVER_NAME, THIS_MODULE, NULL);
#endif
	if (ret < 0) {
		ERR("dvb_register_adapter() failed. ret = %d", ret);
		goto err_kfree;
	}

	/* dvb - dvb_demux */
	device->demux.filternum = 3;
	device->demux.feednum = 3;
	device->demux.start_feed = dvbstub1_start_feed;
	device->demux.stop_feed = dvbstub1_stop_feed;
	device->demux.dmx.capabilities = (
		DMX_TS_FILTERING | 
		DMX_SECTION_FILTERING | 
		DMX_MEMORY_BASED_FILTERING);

	ret = dvb_dmx_init(&device->demux);
	if (ret < 0) {
		ERR("dvb_dmx_init() failed. ret = %d", ret);
		goto err_dvb_unregister_adapter;
	}

	/* dvb - dmxdev */
	device->frontend_hw.source = DMX_FRONTEND_0;
	device->frontend_mem.source = DMX_MEMORY_FE;
	device->dmxdev.filternum = 3;
	device->dmxdev.capabilities = DMXDEV_CAP_DUPLEX;
	device->dmxdev.demux = &device->demux.dmx;
	
	ret = dvb_dmxdev_init(&device->dmxdev, &device->adapter);
	if (ret < 0) {
		ERR("dvb_dmxdev_init() failed. ret = %d", ret);
		goto err_dvb_dmx_release;
	}

	ret = device->demux.dmx.add_frontend(&device->demux.dmx, &device->frontend_hw);
	if (ret < 0) {
		ERR("add_frontend() failed. ret = %d", ret);
		goto err_dvb_dmxdev_release;
	}

	ret = device->demux.dmx.add_frontend(&device->demux.dmx, &device->frontend_mem);
	if (ret < 0) {
		ERR("add_frontend() failed. ret = %d", ret);
		goto err_remove_frontend_hw;
	}

	ret = device->demux.dmx.connect_frontend(&device->demux.dmx, &device->frontend_hw);
	if (ret < 0) {
		ERR("connect_frontend() failed. ret = %d", ret);
		goto err_remove_frontend_mem;
	}

	/* Registering stub frontend */

	ret = dvbstub1_fe_init(&device->frontend);
	if(ret < 0) {
		ERR("dvbstub1_fe_init() failed. ret = %d", ret);
		goto err_disconnect_frontend_hw;
	}

	ret = dvb_register_frontend(&device->adapter, &device->frontend);
	if (ret < 0) {
		ERR("dvb_register_frontend() failed. ret = %d", ret);
		if (device->frontend.ops.release)
			device->frontend.ops.release(&device->frontend);
		goto err_fe_uninit;
	}

	g_dvbstub1 = device;

	ret = dvbstub1_input_init(10, &g_dvbstub_input_ops);
	if(ret < 0) {
		ERR("dvbstub1_input_init() fails. Error code: %d", ret);
		goto err_unregister_frontend;
	}

out:
	DBG("leaving");
	return ret;

/*err_input_cleanup:*/
/*    dvbstub1_input_uninit();*/
err_unregister_frontend:
	dvb_unregister_frontend(&device->frontend);
err_fe_uninit:
	dvbstub1_fe_uninit(&device->frontend);
err_disconnect_frontend_hw:
	device->demux.dmx.disconnect_frontend(&device->demux.dmx);
err_remove_frontend_mem:
	device->demux.dmx.remove_frontend(&device->demux.dmx, &device->frontend_mem);
err_remove_frontend_hw:
	device->demux.dmx.remove_frontend(&device->demux.dmx, &device->frontend_hw);
err_dvb_dmxdev_release:
	dvb_dmxdev_release(&device->dmxdev);
err_dvb_dmx_release:
	dvb_dmx_release(&device->demux);
err_dvb_unregister_adapter:
	dvb_unregister_adapter(&device->adapter);
err_kfree:
	kfree(device);
	DBG("Initialization failed");
	goto out;
}


static void __exit dvbstub1_exit(void)
{
	struct dvbstub1 *device = g_dvbstub1;
	DBG("entering");

	dvbstub1_input_uninit();

	dvb_unregister_frontend(&device->frontend);
	dvbstub1_fe_uninit(&device->frontend);
	device->demux.dmx.close(&device->demux.dmx);
	device->demux.dmx.disconnect_frontend(&device->demux.dmx);
	device->demux.dmx.remove_frontend(&device->demux.dmx, &device->frontend_mem);
	device->demux.dmx.remove_frontend(&device->demux.dmx, &device->frontend_hw);
	dvb_dmxdev_release(&device->dmxdev);
	dvb_dmx_release(&device->demux);
	dvb_unregister_adapter(&device->adapter);
	kfree(device);

	g_dvbstub1 = NULL;

	DBG("leaving");
}

module_init(dvbstub1_init);
module_exit(dvbstub1_exit);



MODULE_AUTHOR("Mironov Sergey <ierton@gmail.com>");

MODULE_DESCRIPTION("Proxy DVB adapter for debuging purposes");

MODULE_LICENSE("GPL");

module_param_named(baud_rate, g_baud_rate, int, 0);

